@@ -66,7 +66,7 @@ pub enum ElfError {
     #[error("Multiple text sections, consider removing llc option: -function-sections")]
     MultipleTextSections,
     /// Read-write data not supported
-    #[error(".bss section not supported")]
+    #[error("Found .bss section in ELF, read-write data not supported")]
     BssNotSupported,
     /// Relocation failed, no loadable section contains virtual address
     #[error("Relocation failed, no loadable section contains virtual address {0:#x}")]
@@ -496,7 +496,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
 
         for section_header in elf.section_headers.iter() {
             if let Some(Ok(this_name)) = elf.shdr_strtab.get(section_header.sh_name) {
-                if this_name == ".bss" {
+                if this_name.starts_with(".bss") {
                     return Err(ElfError::BssNotSupported);
                 }
             }
@@ -649,10 +649,19 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
             if symbol.st_info & 0xEF != 0x02 {
                 continue;
             }
-            let target_pc = symbol.st_value as usize / ebpf::INSN_SIZE - ebpf::ELF_INSN_DUMP_OFFSET;
-            if let Some(Ok(name)) = elf.strtab.get(symbol.st_name) {
-                register_bpf_function(bpf_functions, target_pc, &name)?;
+            if !text_section
+                .vm_range()
+                .contains(&(symbol.st_value as usize))
+            {
+                return Err(ElfError::OutOfBounds);
             }
+            let target_pc = (symbol.st_value - text_section.sh_addr) as usize / ebpf::INSN_SIZE;
+            let name = elf
+                .strtab
+                .get(symbol.st_name)
+                .ok_or(ElfError::UnknownSymbol(symbol.st_name))?
+                .map_err(|_| ElfError::UnknownSymbol(symbol.st_name))?;
+            register_bpf_function(bpf_functions, target_pc, &name)?;
         }
 
         Ok(())
